/*
  This file is part of Simulation-Project:2D Rigid Body Collisions.
 

    Simulation-Project:2D Rigid Body Collisions is free software: you can redistribute 
    it and/or modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Simulation-Project:2D Rigid Body Collisions is distributed in 
    the hope that it will be useful, but WITHOUT ANY WARRANTY;
     without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Simulation-Project:2D Rigid Body Collisions.  
    If not, see <http://www.gnu.org/licenses/>.
*/
package ca.uoit.csci3010u.project;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Random;

import org.opensourcephysics.controls.AbstractSimulation;
import org.opensourcephysics.controls.SimulationControl;
import org.opensourcephysics.display.Trail;
import org.opensourcephysics.frames.PlotFrame;


public class RigidBodyCollisions extends AbstractSimulation{
	public static void main(final String args[]){SimulationControl.createApp(new RigidBodyCollisions());}

	static int max_x;
	static int max_y;
	
	PlotFrame frame = new PlotFrame("X", "Y", "Rigid Body System");
	private double min_radius,max_radius,min_starting_velocity,max_starting_velocity,min_starting_angular_velocity,max_starting_angular_velocity;
	private int number_of_shapes,min_sides,max_sides,max_horizontal,max_vertical;
	
	

	ArrayList<Polygon> shapes;
	public RigidBodyCollisions(){
		frame.setSquareAspect(true);
		frame.setPreferredSize(new Dimension(756, 800));
	}
	private void createShapes(){
		Random rnd=new Random();
		shapes=new ArrayList<Polygon>();
		frame.clearDrawables();
		for(int x=0;x<number_of_shapes;x++){
			final double r=new Random().nextDouble()*(max_radius-min_radius)+min_radius;
			final int margin=(int)Math.ceil(r);
			int temp=max_horizontal-margin*2;
			

			int pX=rnd.nextInt(temp)+margin;
			int pY=rnd.nextInt(max_vertical-margin*2)+margin;
			
			while(tooCloseToNeighbors(pX, pY, r) == true){
				pX=rnd.nextInt(temp)+margin;
				pY=rnd.nextInt(max_vertical-margin*2)+margin;
			}

			final Point c=new Point(pX,pY);
			int l;
			if(max_sides-min_sides>0)
				l=rnd.nextInt(max_sides-min_sides+1)+min_sides;
			else
				l=min_sides;
			
			/*Add random starting velocities*/
			double xV=rnd.nextDouble()*(max_starting_velocity-min_starting_velocity)+min_starting_velocity;
			double yV=rnd.nextDouble()*(max_starting_velocity-min_starting_velocity)+min_starting_velocity;
			int velocity_selector=rnd.nextInt(4);
			if(velocity_selector==0){
				xV*=-1.0;
				yV*=-1.0;
			}else if(velocity_selector==1){
				xV*=-1.0;
			}else if(velocity_selector==2){
				yV*=-1.0;
			}
			
			/*Add random angle*/
			final double startRotation=rnd.nextDouble()*Math.PI*2.0;
			double startAngularVelocity=rnd.nextDouble()*(max_starting_angular_velocity-min_starting_angular_velocity)+min_starting_angular_velocity;
			if(rnd.nextBoolean())
				startAngularVelocity*=-1.0;
			shapes.add(new Polygon(c,l,r,xV,yV, startRotation,startAngularVelocity));
			
			
			draw();

		}
	}
	
	private boolean tooCloseToNeighbors(int x, int y, double r){
		
		for(Polygon s: shapes){
			
			double spacing = Math.sqrt((Math.pow(x - s.center.x, 2) + Math.pow(y - s.center.y, 2)));
			
			if(spacing < r + s.radius){
				return true;
			}
			
		}
		
		return false;
	}
	
	private void draw(){
		frame.clearDrawables();
		
		Trail border = new Trail();
		border.addPoint(0, 0);
		border.addPoint(max_horizontal, 0);
		border.addPoint(max_horizontal, max_vertical);
		border.addPoint(0, max_vertical);
		border.closeTrail();
		frame.addDrawable(border);
		for(int x=0;x<shapes.size();x++){
			Trail trail=new Trail();
			for(final Point p:shapes.get(x).points){
				trail.addPoint(p.x, p.y);
			}
			trail.closeTrail();
			frame.addDrawable(trail);
		}
	}
	
	@Override
	protected void doStep() {
		for(int i=0;i<shapes.size();i++){
			shapes.get(i).center.x+=shapes.get(i).xVelocity;
			shapes.get(i).center.y+=shapes.get(i).yVelocity;
			shapes.get(i).rotation+=shapes.get(i).angularVelocity;
			shapes.get(i).createPoints();
			if(shapes.get(i).getMinX() <= 0){
				shapes.get(i).xVelocity = Math.abs(shapes.get(i).xVelocity);
				shapes.get(i).center.x += 0 - shapes.get(i).getMinX();
				shapes.get(i).angularVelocity*=-1.0;
				shapes.get(i).lastHitShape = -1;
			}else if(shapes.get(i).getMaxX() >= max_x){
				shapes.get(i).xVelocity = Math.abs(shapes.get(i).xVelocity) * -1;
				shapes.get(i).center.x -= shapes.get(i).getMaxX() - RigidBodyCollisions.max_x;
				shapes.get(i).angularVelocity*=-1.0;
				shapes.get(i).lastHitShape = -1;
			}
			
			if(shapes.get(i).getMinY() <= 0){
				shapes.get(i).yVelocity = Math.abs(shapes.get(i).yVelocity);
				shapes.get(i).center.y += 0 - shapes.get(i).getMinY();
				shapes.get(i).angularVelocity*=-1.0;
				shapes.get(i).lastHitShape = -1;
			}else if(shapes.get(i).getMaxY() >= max_y){
				shapes.get(i).yVelocity = Math.abs(shapes.get(i).yVelocity) * -1;
				shapes.get(i).center.y -= shapes.get(i).getMaxY() - RigidBodyCollisions.max_y;
				shapes.get(i).angularVelocity*=-1.0;
				shapes.get(i).lastHitShape = -1;
			}
			shapes.get(i).createPoints();
			
			for(int w=i+1;w<shapes.size();w++){// Check collisions in here
				if(shapes.get(i).checkCollision(shapes.get(w))){
					
					//Collision response in here
					//This code works because all the shapes have the same mass
					double u1x=shapes.get(i).xVelocity;
					double u1y=shapes.get(i).yVelocity;
					double u2x=shapes.get(w).xVelocity;
					double u2y=shapes.get(w).yVelocity;
					double m1=shapes.get(i).mass;
					double m2=shapes.get(w).mass;
					shapes.get(i).xVelocity=((m1-m2)/(m1+m2))*u1x+((2.0*m2)/(m1+m2))*u2x;
					shapes.get(i).yVelocity=((m1-m2)/(m1+m2))*u1y+((2.0*m2)/(m1+m2))*u2y;
					
					shapes.get(w).xVelocity=((m2-m1)/(m1+m2))*u2x+((2.0*m1)/(m1+m2))*u1x;
					shapes.get(w).yVelocity=((m2-m1)/(m1+m2))*u2y+((2.0*m1)/(m1+m2))*u1y;
					
					//Angular velocity
					double u1a=shapes.get(i).angularVelocity;
					double u2a=shapes.get(w).angularVelocity;
					shapes.get(i).angularVelocity=((m1-m2)/(m1+m2))*u1a+((2.0*m2)/(m1+m2))*u2a;
					shapes.get(w).angularVelocity=((m2-m1)/(m1+m2))*u2a+((2.0*m1)/(m1+m2))*u1a;
					
					//Remove penetrations
					if(shapes.get(i).lastHitShape==w){
						double deltaX=Math.abs(shapes.get(i).center.x-shapes.get(w).center.x);
						double deltaY=Math.abs(shapes.get(i).center.y-shapes.get(w).center.y);
						while(shapes.get(i).checkCollision(shapes.get(w))){
							if(shapes.get(i).center.x>shapes.get(w).center.x){
								shapes.get(i).center.x+=0.01;
								shapes.get(w).center.x-=0.01;
							}else{
								shapes.get(i).center.x-=0.01;
								shapes.get(w).center.x+=0.01;
							}
							
							if(shapes.get(i).center.y>shapes.get(w).center.y){
								shapes.get(i).center.y+=0.01 * deltaY / deltaX;
								shapes.get(w).center.y-=0.01* deltaY / deltaX;
							}else{
								shapes.get(i).center.y-=0.01* deltaY / deltaX;
								shapes.get(w).center.y+=0.01* deltaY / deltaX;
							}
							shapes.get(i).createPoints();
							shapes.get(w).createPoints();
						}
					}else{
						shapes.get(i).lastHitShape=w;
					}
				}
			}
		}
		draw();
	}
	
	public void initialize(){
		number_of_shapes=control.getInt("Number of Shapes");
		
		min_sides=control.getInt("Min Sides");
		max_sides=control.getInt("Max Sides");
				
		if(min_sides>max_sides){
			control.println("The max sides was less than the min! Corrected.");
			control.setValue("Max Sides", min_sides);
			max_sides=min_sides;
		}
		min_radius=control.getDouble("Min Radius");
		max_radius=control.getDouble("Max Radius");
		
		if(max_radius < min_radius){
			max_radius = min_radius;
			control.setValue("Max Radius", max_radius);
			control.println("The minimum radius was less than the minimum! Corrected.");
		}
		
		min_starting_velocity=control.getDouble("Min Starting Velocity");
		max_starting_velocity=control.getDouble("Max Starting Velocity");
		
		if(max_starting_velocity < min_starting_velocity){
			max_starting_velocity = min_starting_velocity;
			control.setValue("Max Starting Velocity", max_starting_velocity);
			control.println("Max starting velocity lower than min! Corrected.");
		}
		
		min_starting_angular_velocity=control.getDouble("Min Starting Angular Velocity");
		max_starting_angular_velocity=control.getDouble("Max Starting Angular Velocity");
		
		if(max_starting_angular_velocity < min_starting_angular_velocity){
			max_starting_angular_velocity = min_starting_angular_velocity;
			control.setValue("Max Starting Angular Velocity", max_starting_angular_velocity);
			control.println("Max starting angular velocity was lower than the minimum! Corrected.");
		}
		
		max_horizontal=control.getInt("Max Horizontal");
		max_vertical=control.getInt("Max Vertical");
		
		//Make sure that the frame can contain the largest possible shape
		if(max_radius*2.0>=max_horizontal){
			max_horizontal=(int)Math.ceil(max_radius*2.0)+5;
			control.println("Max horizontal has been increased to accomodate max radius.");
			control.setValue("Max Horizontal", max_horizontal);
		}
		if(max_radius*2.0>=max_vertical){
			max_vertical=(int)Math.ceil(max_radius*2.0)+5;
			control.println("Max vertical has been increased to accomodate max radius.");
			control.setValue("Max Vertical", max_vertical);
		}
		frame.setPreferredMinMax(0.0, max_horizontal, 0.0, max_vertical);
		max_x = max_horizontal;
		max_y = max_vertical;

		createShapes();

	}

	@Override
	public void reset(){
		control.setValue("Number of Shapes", 50);
		control.setValue("Min Radius", 1.0);
		control.setValue("Max Radius", 3.0);
		control.setValue("Min Sides",3);
		control.setValue("Max Sides",5);
		control.setValue("Min Starting Velocity",0.0);
		control.setValue("Max Starting Velocity",0.4);
		control.setValue("Min Starting Angular Velocity",0.0);
		control.setValue("Max Starting Angular Velocity",0.4);
		control.setValue("Max Horizontal",100);
		control.setValue("Max Vertical",100);
	}
}



